import click

from log_driver.init.init_drive_db import initialize_database
from log_driver.init.load_config_to_env import load_config


@click.group()
def main():
    pass


@main.command()
@click.option('--file', type=click.Path(exists=True), required=True, help="Path to the YAML configuration file.")
def load_config_to_env(file):
    """Load configuration file and set environment variables."""
    try:
        load_config(file)
        click.echo("Configuration loaded log drive db config successfully.")
    except Exception as e:
        click.echo(f"Error: {e}")


@main.command()
def init_drive_db():
    """Initialize the database with the configuration from environment variables."""
    try:
        initialize_database()
        click.echo("Log drive database initialized successfully.")
    except Exception as e:
        click.echo(f"Error: {e}")


if __name__ == '__main__':
    main()



import datetime


def date_str2date(date_str: str) -> datetime.date:
    return datetime.datetime.strptime(date_str, "%Y-%m-%d").date()


def get_now_date() -> datetime.date:
    return datetime.datetime.now().date()



from datetime import datetime, timedelta, date
from typing import List, Tuple

from common.db import get_dataframe
from common.time_util import get_now_date


def _get_successful_log_for_time_window_1_param_day(etl_name: str) -> List[date]:
    """
    获取log表中已执行成功的日期list
    :param etl_name:
    :return:
    """

    sql_str = f"""
        SELECT drive_value
        FROM log_drive_table
        WHERE etl_name = '{etl_name}' and etl_result = 1
        order by drive_value;
    """

    result_df = get_dataframe(sql_str)

    def str_to_date(date_str):
        return datetime.strptime(date_str, "%Y-%m-%d").date()

    result_list = result_df['drive_value'].apply(str_to_date).tolist()

    return result_list


def _get_successful_log_for_time_window_2_param(etl_name: str) -> List[Tuple[datetime, datetime]]:
    """
    获取log表中已执行成功的时间序列list
    :param etl_name:
    :return:
    """

    sql_str = f"""
        SELECT drive_value
        FROM log_drive_table
        WHERE etl_name = '{etl_name}' and etl_result = 1
        order by drive_value;
    """

    result_df = get_dataframe(sql_str)

    def str_to_tuple(date_str):
        return tuple([datetime.strptime(tmp.strip(), '%Y-%m-%d %H:%M:%S') for tmp in date_str.split(',')])

    result_list = result_df['drive_value'].apply(str_to_tuple).tolist()
    return result_list


def _get_all_date_list_for_time_window_1_param_day(start_date: date, date_offset: int) -> List[date]:
    """

    :param start_date: 本次数据开始时间
    :param date_offset: 返回的date list的最大date和当前日期的差值，默认差一天.如，今天是04月04日，则返回的date list最大值为04月03.
    :return:
    """

    # 获取当前日期
    now_date = get_now_date()

    # 获取最后日期
    end_date = now_date + timedelta(days=-date_offset)

    # 获取开始时间到当前时间的天数
    interval_days = (now_date - start_date).days

    # 生成日期列表
    date_list = []
    for i in range(interval_days):
        # 计算偏移后的日期
        current_date = start_date + timedelta(days=i)

        # 如果偏移后的日期超过了当前日期，则停止生成
        if current_date > end_date:
            break

        date_list.append(current_date)

    return date_list


def get_sorted_unexecuted_for_time_window_1_param_day(etl_name: str = None, start_date: date = None,
                                                      date_offset: int = 1) -> List[date]:
    """
    日志驱动类型：time_window_1_param_day
    :param etl_name: ETL 名称
    :param start_date: ETL对应数据的初始化时间
    :param date_offset: 日期偏移量，返回的date list的最大date和当前日期的差值，默认差一天.如，今天是04月04日，则返回的date list最大值为04月03.
    :return: 返回待处理日期的list
    """
    if etl_name is None:
        raise ValueError("No parameter etl_name was passed.")
    if start_date is None:
        raise ValueError("No parameter start_date_str was passed.")

    executed_successful_date_list = _get_successful_log_for_time_window_1_param_day(etl_name)

    all_date_list = _get_all_date_list_for_time_window_1_param_day(start_date, date_offset)

    unprocessed_date_list = [date_temp for date_temp in all_date_list if
                             date_temp not in executed_successful_date_list]
    unprocessed_date_list.sort()
    return unprocessed_date_list


def _get_all_date_list_for_time_window_2_param(start_datetime: datetime, time_interval: int) -> List[Tuple[datetime, datetime]]:
    """
    :param start_datetime: 开始时间
    :param time_interval: 时间间隔，单位为分钟。
    :return: 包含时间间隔的元组列表，每个元组包含开始时间和结束时间。
    """

    # 获取当前时间
    end_datetime = datetime.now()

    # 初始化结果列表
    time_intervals = []

    # 开始时间
    current_datetime = start_datetime

    # 生成时间间隔列表
    while current_datetime < end_datetime:
        # 计算结束时间
        end_interval = current_datetime + timedelta(minutes=time_interval)

        # 如果结束时间超过当前时间，则设置结束时间为当前时间
        if end_interval > end_datetime:
            break

        # 添加时间间隔到结果列表中
        time_intervals.append((current_datetime, end_interval))

        # 更新当前时间为结束时间，以便下一次循环
        current_datetime = end_interval

    return time_intervals


def get_sorted_unexecuted_for_time_window_2_param(etl_name: str = None, start_datetime: datetime = None,
                                                  time_interval: int = 1440) -> List[Tuple[datetime, datetime]]:
    """
    从给定的开始时间开始，以给定的时间间隔生成时间序列。返回未执行的时间序列
    :param etl_name:
    :param start_datetime:
    :param time_interval: 单位分钟。默认1440秒
    :return:
    """
    if etl_name is None:
        raise ValueError("No parameter etl_name was passed.")
    if start_datetime is None:
        raise ValueError("No parameter start_date_str was passed.")

    executed_successful_date_list = _get_successful_log_for_time_window_2_param(etl_name)

    all_date_list = _get_all_date_list_for_time_window_2_param(start_datetime, time_interval)

    unprocessed_date_list = list(set(all_date_list).symmetric_difference(set(executed_successful_date_list)))
    unprocessed_date_list.sort()
    return unprocessed_date_list



from datetime import date, datetime
from typing import List

from log_driver.files import get_sorted_unexecuted_file_list
from log_driver.time_windows import get_sorted_unexecuted_for_time_window_1_param_day, \
    get_sorted_unexecuted_for_time_window_2_param


def test_get_sorted_unexecuted_for_time_window_1_param_day():
    etl_name = 'A_etl'
    start_date = date(2024, 3, 26)
    result = get_sorted_unexecuted_for_time_window_1_param_day(etl_name, start_date)
    print(result)


def test_get_sorted_unexecuted_for_time_window_2_param():
    etl_name = 'B_etl'
    start_datetime = datetime.strptime('2024-05-01 00:00:00', "%Y-%m-%d %H:%M:%S")
    time_interval = 2880
    result = get_sorted_unexecuted_for_time_window_2_param(etl_name, start_datetime, time_interval)
    print(result)


def test_get_sorted_unexecuted_file_list():
    def get_file_list() -> List[str]:
        return ['a_20240403.xlsx', 'a_20240404.xlsx', 'a_20240405.xlsx']
    etl_name = 'C_etl'
    result = get_sorted_unexecuted_file_list(get_file_list, etl_name)
    print(result)


if __name__ == '__main__':
    test_get_sorted_unexecuted_for_time_window_1_param_day()
    test_get_sorted_unexecuted_for_time_window_2_param()
    test_get_sorted_unexecuted_file_list()



LOG_DRIVE_DDL = """
    CREATE TABLE log_drive_table (
        id INT NOT NULL AUTO_INCREMENT PRIMARY KEY COMMENT '日志唯一标识',
        etl_name VARCHAR(255) NOT NULL COMMENT 'ETL 任务名称',
        target_table_name VARCHAR(255) NOT NULL COMMENT 'ETL 目标表名称',
        drive_type VARCHAR(255) NOT NULL COMMENT '日志驱动类型，目前枚举值包括：time_window_1_param_day,time_window_1_param_week,time_window_2_param,upstream',
        drive_value VARCHAR(255) NOT NULL COMMENT '日志驱动值',
        process_start_time DATETIME NOT NULL COMMENT 'ETL 处理开始时间',
        process_end_time DATETIME NOT NULL COMMENT 'ETL 处理结束时间',
        etl_result INT NOT NULL COMMENT 'ETL 处理结果,目前枚举值包括：0-执行失败；1-执行成功；2-执行中',
        business_time VARCHAR(255) COMMENT '本批次数据涉及到的业务日期，逗号分割'
    ) COMMENT='日志驱动表';
"""


import os
import platform

import yaml


def load_config(file_path):
    if not os.path.isfile(file_path):
        raise FileNotFoundError(f"Configuration file {file_path} does not exist.")

    with open(file_path, 'r') as stream:
        try:
            config = yaml.safe_load(stream)
            for key, value in config.items():
                os.environ[key] = str(value)
                # 根据操作系统类型设置永久环境变量
                set_permanent_env_var(key, str(value))
        except yaml.YAMLError as exc:
            raise ValueError(f"Error parsing YAML file: {exc}")


def set_permanent_env_var(key, value):
    system = platform.system()
    shell_profile_path = get_shell_profile_path(system)
    with open(shell_profile_path, 'r') as f:
        lines = f.readlines()

    # 检查是否已经存在相同的配置
    existing_line = None
    for i, line in enumerate(lines):
        if f'export {key}=' in line:
            existing_line = i
            break

    if existing_line is not None:
        # 如果已经存在相同的配置，则覆盖原有的配置
        lines[existing_line] = f'export {key}={value}\n'
    else:
        # 否则，添加新的配置
        lines.append(f'export {key}={value}\n')

    # 将更新后的配置写回到文件中
    with open(shell_profile_path, 'w') as f:
        f.writelines(lines)


def get_shell_profile_path(system):
    # 获取当前 shell 的配置文件路径
    if system == "Windows":
        return None  # Windows 暂时不支持永久环境变量
    elif system == "Darwin":
        return get_mac_shell_profile_path()
    elif system == "Linux":
        return get_linux_shell_profile_path()
    else:
        raise ValueError("Unsupported operating system.")


def get_mac_shell_profile_path():
    # 获取 Linux 操作系统下的 shell 配置文件路径
    shell = os.environ.get('SHELL', '')
    if "zsh" in shell:
        return os.path.expanduser('~/.zshrc')
    elif "bash" in shell:
        return os.path.expanduser('~/.bash_profile')
    else:
        raise ValueError("Unsupported shell.")


def get_linux_shell_profile_path():
    # 获取 Linux 操作系统下的 shell 配置文件路径
    shell = os.environ.get('SHELL', '')
    if "zsh" in shell:
        return os.path.expanduser('~/.zshrc')
    elif "bash" in shell:
        return os.path.expanduser('~/.bashrc')
    else:
        raise ValueError("Unsupported shell.")



from datetime import datetime, date
from typing import Tuple, List

from common.db import get_dataframe


def _get_successful_log_for_upstream(etl_name: str) -> List[int]:
    sql_str = f"""
        SELECT drive_value
        FROM log_drive_table
        WHERE etl_name = '{etl_name}' and etl_result = 1
        order by drive_value;
    """

    result_df = get_dataframe(sql_str)

    def get_upstream_id(x):
        return int(x.split('|')[1].strip())

    result_list = result_df['drive_value'].apply(get_upstream_id).tolist()

    return result_list


def _get_all_successful_job_list_for_upstream(upstream_etl_name: str) -> List[int]:
    sql_str = f"""
            SELECT id
            FROM log_drive_table
            WHERE etl_name = '{upstream_etl_name}' and etl_result = 1
            order by drive_value;
        """

    result_df = get_dataframe(sql_str)
    result_list = result_df['id'].apply(lambda x: int(x)).tolist()

    return result_list


def _get_unprocessed_file_list_by_upstream_id(unprocessed_file_list) -> List[Tuple[int, List[date]]]:
    ids_str = ', '.join(map(str, unprocessed_file_list))
    sql_str = f"SELECT * FROM log_drive_table WHERE id IN ({ids_str})"
    result_df = get_dataframe(sql_str)

    def get_business_time(x):
        business_time_list = x['business_time'].split(',')
        business_date_list = [datetime.strptime(date_str, "%Y-%m-%d").date() for date_str in business_time_list.strip()]
        return tuple([x['id'], business_date_list])

    result_list = result_df.apply(get_business_time, axis=1).tolist()
    return result_list


def get_sorted_unexecuted_upstream_job_list(etl_name: str, upstream_etl_name: str) -> List[Tuple[int, List[date]]]:
    executed_successful_file_list = _get_successful_log_for_upstream(etl_name)

    all_successful_job_list = _get_all_successful_job_list_for_upstream(upstream_etl_name)

    unprocessed_file_list = list(set(all_successful_job_list).symmetric_difference(set(executed_successful_file_list)))
    unprocessed_file_list.sort()

    result = _get_unprocessed_file_list_by_upstream_id(unprocessed_file_list)

    return unprocessed_file_list


if __name__ == '__main__':
    get_sorted_unexecuted_upstream_job_list('D_etl', 'A_etl')



from typing import Callable, List

from common.db import get_dataframe


def _get_successful_log_for_files(etl_name: str) -> List[str]:
    """
    获取log表中已执行成功的日期list
    :param etl_name:
    :return:
    """

    sql_str = f"""
        SELECT drive_value
        FROM log_drive_table
        WHERE etl_name = '{etl_name}' and etl_result = 1
        order by drive_value;
    """

    result_df = get_dataframe(sql_str)

    result_list = result_df['drive_value'].tolist()

    return result_list


def get_sorted_unexecuted_file_list(get_file_list_from_file_systems: Callable[..., List[str]], etl_name: str, *args,
                                    **kwargs) -> List[str]:
    """
    获取待执行的文件名称列表
    :param get_file_list_from_file_systems:
    :param etl_name:
    :param args:
    :param kwargs:
    :return:
    """
    executed_successful_file_list = _get_successful_log_for_files(etl_name)
    all_file_list = get_file_list_from_file_systems(*args, **kwargs)
    unprocessed_file_list = list(set(all_file_list).symmetric_difference(set(executed_successful_file_list)))
    unprocessed_file_list.sort()
    return unprocessed_file_list



import json

import setuptools


def parse_pipfile_lock():
    with open('Pipfile.lock') as f:
        lock_data = json.load(f)
    dependencies = lock_data['default']
    return [f"{pkg}{info.get('version', '')}" for pkg, info in dependencies.items()]


setuptools.setup(
    name='nimbletl',
    version='0.1.8',
    packages=setuptools.find_packages(),
    install_requires=parse_pipfile_lock(),
    entry_points={
        'console_scripts': [
            'nimbletl=log_driver.init.cli:main',
        ],
    },
    author="Ma Xiaoqiang",
    author_email="851788096@qq.com",
    description="Make your ETL easier and more nimble",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown'
)



import os

import pandas as pd
from sqlalchemy import create_engine
from sqlalchemy.exc import SQLAlchemyError


def _get_drive_db_url():
    db_user = os.getenv('LOG_DRIVE_DB_USER', 'root')
    db_password = os.getenv('LOG_DRIVE_DB_PASSWORD', '')
    db_host = os.getenv('LOG_DRIVE_DB_HOST', 'localhost')
    db_port = os.getenv('LOG_DRIVE_DB_PORT', '3306')
    db_name = os.getenv('LOG_DRIVE_DB_NAME', 'nimbletl')
    return f'mysql+pymysql://{db_user}:{db_password}@{db_host}:{db_port}/{db_name}'


def _get_engine():
    return create_engine(_get_drive_db_url())


def get_dataframe(sql_str):
    try:
        # 使用同一个数据库连接对象
        engine = _get_engine()
        df = pd.read_sql_query(sql_str, con=engine.connect())
        return df
    except SQLAlchemyError as e:
        print(f"Error occurred: {e}")
    finally:
        if engine:
            engine.dispose()


def execute_sql(sql_str):
    try:
        # 使用同一个数据库连接对象
        engine = _get_engine()
        with engine.connect() as connection:
            connection.execute(sql_str)
        print("Table created successfully.")
    except SQLAlchemyError as e:
        print(f"Error occurred: {e}")
    finally:
        if connection:
            connection.close()
